/* interOps.c */
/* Operationen zur linearen Interpolation. A. Christidis */
#include "interOps.h"

/*************************************************************************/
  int set_iLerp (int Dx, int y0, int yn, int *yStore)
/*************************************************************************/
/*Linear interger interpolation over the number of <Dx> samples between 
  the values <y0> and <yn> after the Bresenham line algrotithm. 
  The resulting <Dx> values are stored in the array <yStore>. */
{ int dx, dy, Dy, dv, incrX, incrY;
  int x=0, y=y0, incr=1;

  if (Dx < 2)  return KO; 
#ifdef  MORE_LERP
  if (yn < y0) incr = -incr; 
#endif//MORE_LERP

  dy = abs(yn-y0); dx = Dx-1; /*Distances betw. first & last sample*/
  Dy = dy+1;                  /*Number of different values to store*/

  if (dx > dy)
  { incrX=2*Dx; incrY=2*Dy;
    dv = -incrX; /*Decision variable for y changes*/
    for (x=0; x<Dx; x++)
    { yStore[x]=y;
      dv += incrY; 
      if (dv >= -Dy) { y += incr;  dv -= incrX; }
    }
  } 
#ifdef  MORE_LERP
else 
  { incrX=2*dx; incrY=2*dy;
    dv=0; /*Decision variable for x changes*/
    for (x=0; x<Dx; x++)
    { yStore[x]=y;
      dv+=incrY; 
      while (dv >= dx) { y += incr;  dv -= incrX; } 
    }
  }
#endif//MORE_LERP

#ifdef  DEBUG_LERP 
  if (yStore[0] != y0 || yStore[dx] != yn)
  { printf ("\rERROR IN set_iLerp!  (CR to continue) "); getchar(); }
#endif//DEBUG_LERP
  return OK;
}

#ifdef  NOTIZEN
/*Benoetigte Oktanten: 1., 2., 7., 8.
  Unterschiedliche Werte = Anzahl Stufen: s=min (Dx, Dy); 
  Stufen-Hoehe: Dy/s (uneinheitlich); Stufen-Laenge: Dx/s (uneinheitlich))
*/
#endif//NOTIZEN

/*************************************************************************/
  float lerp (float a, float b, float t)
/*************************************************************************/
/*Yields fraction t (t=0 ... 1) on the way from a to b (F.S.Hill, p. 160)*/
{ return a + (b - a) * t;
}

#ifdef  WITH_GLEVEL
/*************************************************************************/
  int ilerp (int first, int last, float t)
/*************************************************************************/
/*Yields the linearly partitioned integer closest to the fraction <t> 
  (t=0...1) of the way from value <first> to value <last>*/
{
#ifdef  MORE_LERP
 int len=0, value=0;
  if (t < 0.f) t = -t;
  len = last-first; if (len < 0) len--; else len++;
  value = first+(int)(t*len); 
  /*Ergebnis soll zwischen min und max liegen:*/
  value = MAX (value, MIN(first, last)); //d.h.: >= min
  value = MIN (value, MAX(first, last)); //d.h.: <= max
  return (value);
#endif//MORE_LERP
}

/*************************************************************************/
  int gLevel (float intens, int nLevels)
/*************************************************************************/
/* Assign grey level (index 0...nLevels-1) to light intensity (0 ... 1) */
{ return (ilerp (0, nLevels-1, intens));
}
#endif//WITH_GLEVEL


